#import "Hoppeta.h"

#import "GNGonyNenga.h"
#import "GNScene.h"
#import "GNActor.h"
#import "GNActorEnumerator.h"
#import "GNActorList.h"
#import "GNCamera.h"

@implementation GNCamera
- (id)init
{
	[super init];
	position.x = 0.0f;
	position.y = 3.0f;
	position.z = 0.0f;
	velocity.x = 0.0f;
	velocity.y = 0.0f;
	velocity.z = 0.0f;
	direction = HPIdentityMatrix();

	return self;
}

- (void)process
{
	id logoList;
	id enumerator;
	id logo;
	id foundLogo;
	HPMatrix inversion;
	HPMatrix rotation;
	HPVector targetPosition;
	HPVector distance;
	HPVector invertedDistance;
	HPVector axis;
	float targetZ;
	float rollToDown;
	float rollToUp;
	float roll;
	float pitch;

	logoList = [[GNGonyNenga scene] logoList];
	enumerator = [logoList actorEnumerator];
	foundLogo = nil;
	targetZ = position.z + 100.0f;
	while(logo = [enumerator nextActor])
	{
		if(foundLogo == nil)
		{
		 	foundLogo = logo;
		}
		else
		{
			if(fabs(targetZ - [logo position].z) < fabs(targetZ - [foundLogo position].z))
			{
				foundLogo = logo;
			}
		}
	}

	if(foundLogo != nil)
	{
		targetPosition = [foundLogo position];
		distance.x = targetPosition.x - position.x;
		distance.y = targetPosition.y - position.y;
		distance.z = targetPosition.z - position.z;
		inversion = HPInversionMatrix(direction);
		invertedDistance = HPMultiplyMatrixVector(direction,distance);
		rollToDown = atan2(invertedDistance.x,-invertedDistance.y) * 180.0f / M_PI * 0.1f;
		rollToUp = atan2(-invertedDistance.x,invertedDistance.y) * 180.0f / M_PI * 0.1f;
		if(fabs(rollToDown) < fabs(rollToUp) * 0.5f)
		{
			roll = rollToDown;
		}
		else
		{
			roll = rollToUp;
		}
		axis.x = direction.data[2 * 4 + 0];
		axis.y = direction.data[2 * 4 + 1];
		axis.z = direction.data[2 * 4 + 2];
		rotation = HPRotationMatrix(axis,roll);
		direction = HPMultiplyMatrixes(direction,rotation);
		invertedDistance = HPMultiplyMatrixVector(direction,distance);
		pitch = atan2(-invertedDistance.y,invertedDistance.z) * 180.0f / M_PI * 0.1f;
		axis.x = direction.data[0 * 4 + 0];
		axis.y = direction.data[0 * 4 + 1];
		axis.z = direction.data[0 * 4 + 2];
		rotation = HPRotationMatrix(axis,pitch);
		direction = HPMultiplyMatrixes(direction,rotation);
	}

	velocity.x += direction.data[2 * 4 + 0] * 0.5f;
	velocity.y += direction.data[2 * 4 + 1] * 0.5f;
	velocity.z += direction.data[2 * 4 + 2] * 0.5f;
	velocity.x *= 0.9f;
	velocity.y *= 0.9f;
	velocity.z *= 0.9f;
	position.x += velocity.x;
	position.y += velocity.y;
	position.z += velocity.z;
}

- (void)setView
{
	HPVector viewVector;
	HPVector viewUpVector;

	viewVector.x = direction.data[2 * 4 + 0];
	viewVector.y = direction.data[2 * 4 + 1];
	viewVector.z = direction.data[2 * 4 + 2];
	viewUpVector.x = direction.data[1 * 4 + 0];
	viewUpVector.y = direction.data[1 * 4 + 1];
	viewUpVector.z = direction.data[1 * 4 + 2];

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(
		position.x,
		position.y,
		position.z,
		position.x + viewVector.x,
		position.y + viewVector.y,
		position.z + viewVector.z,
		viewUpVector.x,
		viewUpVector.y,
		viewUpVector.z);
}

- (HPVector)position
{
	return position;
}

- (HPVector)velocity
{
	return velocity;
}

- (HPMatrix)direction
{
	return direction;
}
@end
