#import "Hoppeta.h"

#import "GNTitle.h"
#import "GNScene.h"
#import "GNGonyNenga.h"

static id title;
static id scene;
static GNResource resource;

@implementation GNGonyNenga
- (id)init
{
	[super init];
	if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0)
	{
		fprintf(stderr,"Couldn't initialize SDL: %s\n",SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);

	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);
	screenSurface = SDL_SetVideoMode(640,480,32,SDL_OPENGL | SDL_FULLSCREEN);
	if(screenSurface == NULL)
	{
		fprintf(stderr,"Couldn't set %dx%d video mode: %s\n",640,480,SDL_GetError());
		exit(1);
	}
	SDL_ShowCursor(SDL_DISABLE);

	/// @todo λ򤹤
	if(Mix_OpenAudio(MIX_DEFAULT_FREQUENCY,MIX_DEFAULT_FORMAT,2,1024) < 0)
	{
		fprintf(stderr,"Couldn't initialize mixer: %s\n",SDL_GetError());
	}
	
	resource.joystick = [[HPJoystick alloc] init];
	[self addChildObject:resource.joystick];
	resource.font = [[HPFont alloc] init];
	[self addChildObject:resource.font];
	resource.flashTexture = [[HPTexture alloc] initWithFilename:"Flash.bmp"];
	[self addChildObject:resource.flashTexture];
	resource.starTexture = [[HPTexture alloc] initWithFilename:"Star.bmp"];
	[self addChildObject:resource.starTexture];
	resource.music = [[HPMusic alloc] initWithFilename:"GONYNENGA11.ogg"];
	[self addChildObject:resource.music];
	scene = [[GNScene alloc] init];
	[self addChildObject:scene];
	title = [[GNTitle alloc] init];
	[self addChildObject:title];

	return self;
}

- (float)mainLoop
{
	id currentScene;
	SDL_Event event;
	Sint32 frameNum;
	Sint32 startTicks;
	Sint32 currentTicks;
	Sint32 processStartTicks;
	Sint32 processTicks;
	Sint32 delay;
	BOOL done;
	BOOL sceneResult;

	frameNum = 0;
	currentScene = title;
	startTicks = SDL_GetTicks();

	[currentScene begin];
	done = NO;
	while(done == NO)
	{
		processStartTicks = SDL_GetTicks();
		while(SDL_PollEvent(&event))
		{
			switch(event.type)
			{
			case SDL_MOUSEBUTTONDOWN:
				SDL_WarpMouse(screenSurface->w / 2,screenSurface->h / 2);
				break;

			case SDL_KEYDOWN:
				if(event.key.keysym.sym == SDLK_ESCAPE)
				{
					done = YES;
				}
				break;

			case SDL_QUIT:
				done = YES;
				break;

			default:
				break;
			}
		}

		[resource.joystick update];
		sceneResult = [currentScene processReturnsResult];

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		[currentScene draw];
		SDL_GL_SwapBuffers();

		if(sceneResult == YES)
		{
			currentScene = scene;
			[currentScene begin];
		}

		[[HPAutoreleasePool defaultAutoreleasePool] flush];

		processTicks = SDL_GetTicks() - processStartTicks;
		delay = 16 - processTicks;
		if(delay < 0)
		{
			delay = 1;
		}
		SDL_Delay(delay);

		frameNum++;
	}

	currentTicks = SDL_GetTicks();
	if(currentTicks > startTicks)
	{
		return ((double)frameNum * 1000.0f) / (currentTicks - startTicks);
	}
	else
	{
		return -1.0f;
	}
}

+ (id)scene;
{
	return scene;
}

+ (GNResource *)resource;
{
	return &resource;
}
@end
