#import "Hoppeta.h"

#import "GNCamera.h"
#import "GNSphere.h"
#import "GNTypewriterText.h"
#import "GNFader.h"
#import "GNActor.h"
#import "GNActorList.h"
#import "GNActorEnumerator.h"
#import "GNStars.h"
#import "GNLogo.h"
#import "GNGonyNenga.h"
#import "GNScene.h"

static GNTypewriterTextSource greetingTypewriterTextSource[] =
{
	{
		208.0f,
		124.0f,
		(Sint8 *)"HAPPY NEW YEAR",
		NO
	},
	{
		48.0f,
		344.0f,
		(Sint8 *)"WISHING YOU THE VERY BEST NEW YEAR",
		YES
	},
};

@implementation GNScene
- (id)init
{
	[super init];
	camera = [[GNCamera alloc] init];
	[self addChildObject:camera];
	sphere = [[GNSphere alloc] init];
	[self addChildObject:sphere];
	stars = [[GNStars alloc] init];
	[self addChildObject:stars];
 	logoList = [[GNActorList alloc] initWithSize:256];
	[self addChildObject:logoList];
	typewriterText = [[GNTypewriterText alloc] initWithSource:greetingTypewriterTextSource];
	[self addChildObject:typewriterText];
	fader = [[GNFader alloc] init];
	[self addChildObject:fader];

	count = 0;
	logoCharacterIndex = 0;
	logoCharacterCount = 0;
	logoXCount = 0;
	logoXCountMax = 0;
	logoXAmplitude = 0.0f;
	logoYCount = 0;
	logoYCountMax = 0;
	logoYAmplitude = 0.0f;
	logoFromRed = (rand() & 255) / 255.0f;
	logoFromGreen = (rand() & 255) / 255.0f;
	logoFromBlue = (rand() & 255) / 255.0f;
	logoToRed = (rand() & 255) / 255.0f;
	logoToGreen = (rand() & 255) / 255.0f;
	logoToBlue = (rand() & 255) / 255.0f;

	return self;
}

- (void)begin
{
	[[GNGonyNenga resource]->music play];
	[typewriterText begin];
	[fader begin:GNFaderFadeTypeFadeIn colorType:GNFaderColorTypeBlack frameNum:300];
}

- (BOOL)processReturnsResult
{
	HPVector position;
	float ratio;
	float red;
	float green;
	float blue;
	float alpha;

	count++;

	if(count > 240)
	{
		logoCharacterCount++;
		if(logoCharacterCount > 240)
		{
			logoCharacterCount = 0;
			logoCharacterIndex++;
			logoCharacterIndex %= 15;
			logoFromRed = logoToRed;
			logoFromGreen = logoToGreen;
			logoFromBlue = logoToBlue;
			logoToRed = (rand() & 255) / 255.0f;
			logoToGreen = (rand() & 255) / 255.0f;
			logoToBlue = (rand() & 255) / 255.0f;
		}
		logoXCount++;
		if(logoXCount >= logoXCountMax)
		{
			logoXCount = 0;
			logoXCountMax = 120 + (rand() & 255);
			logoXAmplitude = (float)(rand() & 511) - 256.0f;
		}
		logoYCount++;
		if(logoYCount >= logoYCountMax)
		{
			logoYCount = 0;
			logoYCountMax = 120 + (rand() & 255);
			logoYAmplitude = (float)(rand() & 511) - 256.0f;
		}
		position = [camera position];
		position.x = sin(logoXCount / (float)logoXCountMax * 3.14f) * logoXAmplitude;
		position.y = sin(logoYCount / (float)logoYCountMax * 3.14f) * logoYAmplitude;
		position.z += 1000.0f;
		alpha = (count & 31) / 32.0f * 2.0f;
		if(alpha > 1.0f)
		{
			alpha = 2.0f - alpha;
		}
		alpha = alpha * 0.8f + 0.2f;
		ratio = logoCharacterCount / 240.0f;
		red = logoFromRed * (1.0f - ratio) + logoToRed * ratio;
		green = logoFromGreen * (1.0f - ratio) + logoToGreen * ratio;
		blue = logoFromBlue * (1.0f - ratio) + logoToBlue * ratio;
		[logoList addActor:[[[GNLogo alloc] initWithPosition:position characterIndex:logoCharacterIndex red:red * alpha green:green * alpha  blue:blue * alpha] autorelease]];
	}

	if(count == 300)
	{
		[typewriterText fadeOutWithFrameNum:120];
	}

	[camera process];
	[sphere process];
	[stars process];
	[logoList process];
	[typewriterText process];
	[fader process];

	return NO;
}

- (void)draw
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(
		60.0f,
		320.0f / 240.0f,
		0.1f,
		1000.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	[camera setView];
	[sphere draw];
	[stars draw];
	[logoList draw];

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(
		0.0f,
		640.0f,
		480.0f,
		0.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	[fader draw];
	[typewriterText draw];
}

- (id)logoList
{
	return logoList;
}

- (id)camera
{
	return camera;
}

- (void)dealloc
{
	[super dealloc];
}
@end
