#import "Hoppeta.h"

#import "GNCamera.h"
#import "GNScene.h"
#import "GNGonyNenga.h"
#import "GNActor.h"
#import "GNStars.h"

#define GNStarsDistanceMaxX 256.0f
#define GNStarsDistanceMaxY 256.0f
#define GNStarsDistanceMaxZ 512.0f

@implementation GNStars
- (id)init;
{
	GNStar * aStar;
	Sint32 i;

	[super init];
	for(i = 0;i < GNStarsStarNum;i++)
	{
		aStar = &star[i];
		aStar->position.x = (rand() & 511) - 255.0f;
		aStar->position.y = (rand() & 511) - 255.0f;
		aStar->position.z = (rand() & 1023) - 511.0f;
		aStar->scale = ((rand() & 255) + 256) / 32.0f;
		aStar->red = (rand() & 255) / 255.0f;
		aStar->green = (rand() & 255) / 255.0f;
		aStar->blue = (rand() & 255) / 255.0f;
	}

	return self;
}

- (void)process
{
	GNStar * aStar;
	HPVector cameraPosition;
	Sint32 i;

	cameraPosition = [[[GNGonyNenga scene] camera] position];
	for(i = 0;i < GNStarsStarNum;i++)
	{
		aStar = &star[i];
		if((aStar->position.x + GNStarsDistanceMaxX) < cameraPosition.x)
		{
			aStar->position.x += GNStarsDistanceMaxX * 2.0f;
		}
		if((aStar->position.x - GNStarsDistanceMaxX) > cameraPosition.x)
		{
			aStar->position.x -= GNStarsDistanceMaxX * 2.0f;
		}
		if((aStar->position.y + GNStarsDistanceMaxY) < cameraPosition.y)
		{
			aStar->position.y += GNStarsDistanceMaxY * 2.0f;
		}
		if((aStar->position.y - GNStarsDistanceMaxY) > cameraPosition.y)
		{
			aStar->position.y -= GNStarsDistanceMaxY * 2.0f;
		}
		if((aStar->position.z + GNStarsDistanceMaxZ) < cameraPosition.z)
		{
			aStar->position.z += GNStarsDistanceMaxZ * 2.0f;
		}
		if((aStar->position.z - GNStarsDistanceMaxZ * 2.0f) > cameraPosition.z)
		{
			aStar->position.z -= GNStarsDistanceMaxZ * 2.0f;
		}
	}
}

- (void)draw
{
	static float fogColor[4] = 
	{
		0.0f,
		0.0f,
		0.0f,
		1.0f
	};

	id camera;
	GNStar * aStar;
	HPMatrix cameraDirection;
	HPVector cameraPosition;
	HPVector cameraVelocity;
	HPVector offset;
	float roll;
	Sint32 i;

	camera = [[GNGonyNenga scene] camera];
	cameraDirection = [camera direction];
	cameraPosition = [camera position];
	cameraVelocity = [camera velocity];

	glFogi(GL_FOG_MODE,GL_EXP2);
	glFogfv(GL_FOG_COLOR,fogColor);
	glFogf(GL_FOG_DENSITY,0.002f);
	glHint(GL_FOG_HINT,GL_DONT_CARE);
	glEnable(GL_FOG);
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	[[GNGonyNenga resource]->flashTexture bind];
	for(i = 0;i < GNStarsStarNum;i++)
	{
		aStar = &star[i];
		roll = HPDistanceVectors(aStar->position,cameraPosition);
		glPushMatrix();
		glTranslatef(aStar->position.x,aStar->position.y,aStar->position.z);
		glMultMatrixd(cameraDirection.data);
		glRotatef(roll,0.0f,0.0f,1.0f);
		glScalef(aStar->scale,aStar->scale,aStar->scale);
		glBegin(GL_TRIANGLE_STRIP);
		glColor3f(aStar->red,aStar->green,aStar->blue);
		glTexCoord2f(0.0f,0.0f);
		glVertex3f(-0.5f,0.5f,0.0f);
		glColor3f(aStar->red,aStar->green,aStar->blue);
		glTexCoord2f(1.0f,0.0f);
		glVertex3f(0.5f,0.5f,0.0f);
		glColor3f(aStar->red,aStar->green,aStar->blue);
		glTexCoord2f(0.0f,1.0f);
		glVertex3f(-0.5f,-0.5f,0.0f);
		glColor3f(aStar->red,aStar->green,aStar->blue);
		glTexCoord2f(1.0f,1.0f);
		glVertex3f(0.5f,-0.5f,0.0f);
		glEnd();
		glPopMatrix();
	}
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
		
	offset.x = cameraVelocity.x * 10.0f;
	offset.y = cameraVelocity.y * 10.0f;
	offset.z = cameraVelocity.z * 10.0f;
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	glBegin(GL_LINES);
	for(i = 0;i < GNStarsStarNum;i++)
	{
		aStar = &star[i];
		glColor3f(aStar->red,aStar->green,aStar->blue);
		glVertex3f(aStar->position.x,aStar->position.y,aStar->position.z);
		glColor3f(0.0f,0.0f,0.0f);
		glVertex3f(aStar->position.x + offset.x,aStar->position.y + offset.y,aStar->position.z + offset.z);
	}
	glEnd();
	glDisable(GL_BLEND);
	glDisable(GL_FOG);

}
@end
