#import "Hoppeta.h"

#import "GNGonyNenga.h"
#import "GNTypewriterText.h"

@implementation GNTypewriterText
- (id)initWithSource:(GNTypewriterTextSource *)aSource
{
	[super init];
	source = aSource;
	sourceIndex = 0;
	cursorPosition = 0;
	count = 0;
	fadeCount = 0;
	fadeCountMax = 0;
	processing = NO;
	fading = NO;
	done = NO;

	return self;
}

- (void)begin
{
	processing = YES;
}

- (void)end
{
	processing = NO;
}

- (void)fadeOutWithFrameNum:(Sint32)frameNum
{
	fading = YES;
	fadeCount = 0;
	fadeCountMax = frameNum;
}

- (void)process
{
	if(processing == YES)
	{
		if(done == NO)
		{
			cursorPosition++;
			if(source[sourceIndex].text[cursorPosition] == '\0')
			{
				if(source[sourceIndex].terminated == YES)
				{
					done = YES;
				}
				else
				{
					sourceIndex++;
					cursorPosition = 0;
				}
			}
		}
		else
		{
			count++;
		}

		if(fading == YES)
		{
			fadeCount++;
			if(fadeCount >= fadeCountMax)
			{
				processing = NO;
				fading = NO;
			}
		}
	}
}

- (void)draw
{
	id font;
	Sint32 i;
	Sint32 j;
	float x;
	float y;
	float alpha;

	if(processing == YES)
	{
		alpha = 1.0f;
		if(fading == YES)
		{
			alpha = 1.0f - (float)fadeCount / (float)fadeCountMax;
		}

		font = [GNGonyNenga resource]->font;
		for(i = 0;i <= sourceIndex;i++)
		{
			if(done == NO && i == sourceIndex)
			{
				x = source[i].x;
				y = source[i].y;
				for(j = 0;j < cursorPosition;j++)
				{
					glPushMatrix();
					glTranslatef(x,y,0.0f);
					glScalef(2.0f,2.0f,2.0f);
					[font draw:source[i].text[j] red:1.0f green:1.0f blue:1.0f alpha:alpha];
					glPopMatrix();
					x += 16.0f;
				}
				glPushMatrix();
				glTranslatef(x,y,0.0f);
				glEnable(GL_BLEND);
				glBlendFunc(GL_SRC_ALPHA,GL_ONE);
				glBegin(GL_TRIANGLE_STRIP);
				glColor3f(0.5f,0.5f,0.5f);
				glVertex3f(0.0f,0.0f,0.0f);
				glColor3f(0.5f,0.5f,0.5f);
				glVertex3f(16.0f,0.0f,0.0f);
				glColor3f(1.0f,1.0f,1.0f);
				glVertex3f(0.0f,16.0f,0.0f);
				glColor3f(1.0f,1.0f,1.0f);
				glVertex3f(16.0f,16.0f,0.0f);
				glEnd();
				glDisable(GL_BLEND);
				glPopMatrix();
			}
			else
			{
				x = source[i].x;
				y = source[i].y;
				glPushMatrix();
				glTranslatef(x,y,0.0f);
				glScalef(2.0f,2.0f,2.0f);
				[font drawText:source[i].text red:1.0f green:1.0f blue:1.0f alpha:alpha];
				glPopMatrix();
			}
		}

		if(done == YES)
		{
			x = source[sourceIndex].x + cursorPosition * 16.0f;
			y = source[sourceIndex].y;
			alpha = (120 - count) / 120.0f;
			if(alpha < 0.0f)
			{
				alpha = 0.0f;
			}
			if((count & 1) == 1)
			{
				alpha = 0.0f;
			}
			glPushMatrix();
			glTranslatef(x,y,0.0f);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA,GL_ONE);
			glBegin(GL_TRIANGLE_STRIP);
			glColor3f(0.5f * alpha,0.5f * alpha,0.5f * alpha);
			glVertex3f(0.0f,0.0f,0.0f);
			glColor3f(0.5f * alpha,0.5f * alpha,0.5f * alpha);
			glVertex3f(16.0f,0.0f,0.0f);
			glColor3f(1.0f * alpha,1.0f * alpha,1.0f * alpha);
			glVertex3f(0.0f,16.0,0.0f);
			glColor3f(1.0f * alpha,1.0f * alpha,1.0f * alpha);
			glVertex3f(16.0f,16.0f,0.0f);
			glEnd();
			glDisable(GL_BLEND);
			glPopMatrix();
		}
	}
}
@end
