#import "Hoppeta.h"

#import "HPTexture.h"

enum
{
	HPPlatformEndianTypeLittle = 0,
	HPPlatformEndianTypeBig = 1,
};

static int HPPlatformEndian(void);

@implementation HPTexture
- (id)initWithFilename:(char *)aFilename
{
	SDL_Surface * originalSurface;
	SDL_Surface * convertedSurface;
	SDL_Rect rect;
	Uint32 flags;
	Uint8 alpha;
	Uint32 width;
	Uint32 height;

	[super init];
	loaded = NO;
	originalSurface = SDL_LoadBMP(aFilename);
	width = originalSurface->w;
	height = originalSurface->h;

	if(HPPlatformEndian() == HPPlatformEndianTypeLittle)
	{
		convertedSurface = SDL_CreateRGBSurface(
			SDL_SWSURFACE,
			width,height,
			32,
			0x000000FF, 
			0x0000FF00, 
			0x00FF0000, 
			0xFF000000);
	}
	else
	{
		convertedSurface = SDL_CreateRGBSurface(
			SDL_SWSURFACE,
			width,height,
			32,
			0xFF000000,
			0x00FF0000, 
			0x0000FF00, 
			0x000000FF);
	}
	if(convertedSurface == NULL)
	{
		// 
		printf("SDL_CreateRGBSurface failure\n");
		return self;
	}

	flags = originalSurface->flags & (SDL_SRCALPHA | SDL_RLEACCELOK);
	alpha = originalSurface->format->alpha;
	if(flags & SDL_SRCALPHA)
	{
		SDL_SetAlpha(originalSurface,0,0);
	}
	rect.x = 0;
	rect.y = 0;
	rect.w = width;
	rect.h = height;
	SDL_BlitSurface(originalSurface,&rect,convertedSurface,&rect);
	if(flags & SDL_SRCALPHA)
	{
		SDL_SetAlpha(originalSurface,flags,alpha);
	}

	glGenTextures(1,&texture);
	glBindTexture(GL_TEXTURE_2D,texture);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexImage2D(
		GL_TEXTURE_2D,
		0,
		GL_RGBA,
		width,
		height,
		0,
		GL_RGBA,
		GL_UNSIGNED_BYTE,
		convertedSurface->pixels);
	SDL_FreeSurface(originalSurface);
	SDL_FreeSurface(convertedSurface);

	return self;
}

- (void)bind;
{
	glBindTexture(GL_TEXTURE_2D,texture);
}
@end

static int HPPlatformEndian(void)
{
	short testValue = 0x1234;
	char * testValueAddress = (char *)&testValue;
	if(testValueAddress[0] == 0x34)
	{
		return HPPlatformEndianTypeLittle;
	}

	return HPPlatformEndianTypeBig;
}
