#import "Hoppeta.h"

double HPLengthVector(HPVector aVector)
{
	return sqrt(aVector.x * aVector.x + aVector.y * aVector.y + aVector.z * aVector.z);
}

double HPDistanceVectors(HPVector aVector,HPVector bVector)
{
	double distanceX;
	double distanceY;
	double distanceZ;

	distanceX = aVector.x - bVector.x;
	distanceY = aVector.y - bVector.y;
	distanceZ = aVector.z - bVector.z;

	return sqrt(distanceX * distanceX + distanceY * distanceY + distanceZ * distanceZ);
}

HPVector HPNormalizedVector(HPVector aVector)
{
	HPVector result;
	double length;

	length = sqrt(aVector.x * aVector.x + aVector.y * aVector.y + aVector.z * aVector.z);
	result.x = aVector.x / length;
	result.y = aVector.y / length;
	result.z = aVector.z / length;

	return result;
}

HPVector HPCrossProductVectors(HPVector aVector,HPVector bVector)
{
	HPVector result;

	result.x = aVector.y * bVector.z - aVector.z * bVector.y;
	result.y = aVector.z * bVector.x - aVector.x * bVector.z;
	result.z = aVector.x * bVector.y - aVector.y * bVector.x;

	return result;
}

HPVector HPMultiplyMatrixVector(HPMatrix aMatrix,HPVector aVector)
{
	HPVector result;

	result.x = aMatrix.data[0 * 4 + 0] * aVector.x + aMatrix.data[0 * 4 + 1] * aVector.y + aMatrix.data[0 * 4 + 2] * aVector.z;
	result.y = aMatrix.data[1 * 4 + 0] * aVector.x + aMatrix.data[1 * 4 + 1] * aVector.y + aMatrix.data[1 * 4 + 2] * aVector.z;
	result.z = aMatrix.data[2 * 4 + 0] * aVector.x + aMatrix.data[2 * 4 + 1] * aVector.y + aMatrix.data[2 * 4 + 2] * aVector.z;

	return result;
}

HPMatrix HPIdentityMatrix(void)
{
	HPMatrix result;

	memset(result.data,0,sizeof(result.data));
	result.data[0 * 4 + 0] = 1.0f;
	result.data[1 * 4 + 1] = 1.0f;
	result.data[2 * 4 + 2] = 1.0f;
	result.data[3 * 4 + 3] = 1.0f;

	return result;
}

HPMatrix HPInversionMatrix(HPMatrix aMatrix)
{
	HPMatrix result;
	Sint32 i;
	Sint32 j;

	for(i = 0;i < 4;i++)
	{
		for(j = 0;j < 4;j++)
		{
			result.data[i * 4 + j] = aMatrix.data[j * 4 + i];
		}
	}

	return result;
}

HPMatrix HPRotationMatrix(HPVector aVector,double angle)
{
	HPMatrix result;
	double * pdata;
	double length;
	double sine;
	double cosine;
	double onemcosine;
	double x;
	double y;
	double z;
	double xx;
	double yy;
	double zz;
	double xy;
	double yz;
	double zx;
	double xs;
	double ys;
	double zs;

	x = aVector.x;
	y = aVector.y;
	z = aVector.z;
	length = sqrt(x * x + y * y + z * z);
	if(length == 0.0f)
	{
		// ǥХɲ
		result = HPIdentityMatrix();
		return result;
	}

	x /= length;
	y /= length;
	z /= length;
	sine = sin(angle * M_PI / 180.0f);
	cosine = cos(angle * M_PI / 180.0f);
	onemcosine = 1.0f - cosine;
	xx = onemcosine * x * x;
	yy = onemcosine * y * y;
	zz = onemcosine * z * z;
	xy = onemcosine * x * y;
	yz = onemcosine * y * z;
	zx = onemcosine * z * x;
	xs = x * sine;
	ys = y * sine;
	zs = z * sine;

	pdata = result.data;
	*pdata++ = xx + cosine;
	*pdata++ = xy + zs;
	*pdata++ = zx - ys;
	*pdata++ = 0.0f;
	*pdata++ = xy - zs;
	*pdata++ = yy + cosine;
	*pdata++ = yz + xs;
	*pdata++ = 0.0f;
	*pdata++ = zx + ys;
	*pdata++ = yz - xs;
	*pdata++ = zz + cosine;
	*pdata++ = 0.0f;
	*pdata++ = 0.0f;
	*pdata++ = 0.0f;
	*pdata++ = 0.0f;
	*pdata++ = 1.0f;

	return result;
}

HPMatrix HPMultiplyMatrixes(HPMatrix aMatrix,HPMatrix bMatrix)
{
	HPMatrix result;
	double value;
	Sint32 i;
	Sint32 j;
	Sint32 k;

	for(i = 0;i < 4;i++)
	{
		for(j = 0;j < 4;j++)
		{
			value = 0;
			for(k = 0;k < 4;k++)
			{
				value += aMatrix.data[i * 4 + k] * bMatrix.data[k * 4 + j];
			}
			result.data[i * 4 + j] = value;
		}
	}

	return result;
}
